@@ -2,9 +2,10 @@
 * systeminformation.cpp
 * Copyright 2009    Dario Andres Rodriguez <andresbajotierra@gmail.com>
 * Copyright 2009    George Kiagiadakis <gkiagia@users.sourceforge.net>
+* Copyright 2019    Harald Sitter <sitter@kde.org>
 *
-* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
+* This program is free software; you can redistribute it and/or
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
@@ -27,7 +28,6 @@
 # include <sys/utsname.h>
 #endif
 
-#include <QFile>
 
 #include <KProcess>
 #include "drkonqi_debug.h"
@@ -36,20 +36,64 @@
 #include <KSharedConfig>
 #include <kcoreaddons_version.h>
 #include <KCoreAddons>
+#include <KOSRelease>
 #include <QStandardPaths>
 
 static const QString OS_UNSPECIFIED = QStringLiteral("unspecified");
 static const QString PLATFORM_UNSPECIFIED = QStringLiteral("unspecified");
 
-SystemInformation::SystemInformation(QObject * parent)
-    : QObject(parent)
-    , m_bugzillaOperatingSystem(OS_UNSPECIFIED)
-    , m_bugzillaPlatform(PLATFORM_UNSPECIFIED)
+// This function maps the operating system to an OS value that is
+// accepted by bugs.kde.org. If the values change on the server
+// side, they need to be updated here as well.
+static QString fetchBasicOperatingSystem()
 {
-    // NOTE: the relative order is important here
-    m_bugzillaOperatingSystem = fetchOSBasicInformation();
-    m_operatingSystem = fetchOSDetailInformation();
+    // krazy:excludeall=cpp
+    // Get the base OS string (bugzillaOS)
+#if defined(Q_OS_LINUX)
+    return QStringLiteral("Linux");
+#elif defined(Q_OS_FREEBSD)
+    return QStringLiteral("FreeBSD");
+#elif defined(Q_OS_NETBSD)
+    return QStringLiteral("NetBSD");
+#elif defined(Q_OS_OPENBSD)
+    return QStringLiteral("OpenBSD");
+#elif defined(Q_OS_AIX)
+    return QStringLiteral("AIX");
+#elif defined(Q_OS_HPUX)
+    return QStringLiteral("HP-UX");
+#elif defined(Q_OS_IRIX)
+    return QStringLiteral("IRIX");
+#elif defined(Q_OS_OSF)
+    return QStringLiteral("Tru64");
+#elif defined(Q_OS_SOLARIS)
+    return QStringLiteral("Solaris");
+#elif defined(Q_OS_CYGWIN)
+    return QStringLiteral("Cygwin");
+#elif defined(Q_OS_DARWIN)
+    return QStringLiteral("OS X");
+#elif defined(Q_OS_WIN32)
+    return QStringLiteral("MS Windows");
+#else
+    return OS_UNSPECIFIED;
+#endif
+}
 
+SystemInformation::Config::Config()
+    : basicOperatingSystem(fetchBasicOperatingSystem())
+    , lsbReleasePath(QStandardPaths::findExecutable(QStringLiteral("lsb_release")))
+    , osReleasePath(/* Use KOSRelease default */)
+{
+}
+
+SystemInformation::SystemInformation(Config infoConfig, QObject *parent)
+    : QObject(parent)
+    , m_bugzillaOperatingSystem(infoConfig.basicOperatingSystem)
+    , m_bugzillaPlatform(PLATFORM_UNSPECIFIED)
+    , m_complete(false)
+    , m_infoConfig(infoConfig)
+{
+    // NOTE: order matters. These require m_bugzillaOperatingSystem to be set!
+    m_operatingSystem = fetchOSDetailInformation();
     tryToSetBugzillaPlatform();
 
     KConfigGroup config(KSharedConfig::openConfig(), "SystemInformation");
@@ -81,8 +125,8 @@ void SystemInformation::tryToSetBugzillaPlatform()
 void SystemInformation::tryToSetBugzillaPlatformFromExternalInfo()
 {
     //Run lsb_release async
-    QString lsb_release = QStandardPaths::findExecutable(QLatin1String("lsb_release"));
-    if ( !lsb_release.isEmpty() ) {
+    QString lsb_release = m_infoConfig.lsbReleasePath;
+    if (!lsb_release.isEmpty()) {
         qCDebug(DRKONQI_LOG) << "found lsb_release";
         KProcess *process = new KProcess();
         process->setOutputChannelMode(KProcess::OnlyStdoutChannel);
@@ -95,6 +139,7 @@ void SystemInformation::tryToSetBugzillaPlatformFromExternalInfo()
         const QString& osReleaseInfo = fetchOSReleaseInformation();
         const QString& platform = guessBugzillaPlatform(osReleaseInfo);
         setBugzillaPlatform(platform);
+        m_complete = true;
     }
 }
 
@@ -102,11 +147,11 @@ void SystemInformation::lsbReleaseFinished()
 {
     KProcess *process = qobject_cast<KProcess*>(sender());
     Q_ASSERT(process);
-    m_lsbRelease = QString::fromLocal8Bit(process->readAllStandardOutput().trimmed());
+    m_distributionPrettyName = QString::fromLocal8Bit(process->readAllStandardOutput().trimmed());
     process->deleteLater();
 
     //Guess distro string
-    QString platform = guessBugzillaPlatform(m_lsbRelease);
+    QString platform = guessBugzillaPlatform(m_distributionPrettyName);
 
     // if lsb_release doesn't work well, turn to the /etc/os-release file
     if (platform == PLATFORM_UNSPECIFIED) {
@@ -115,6 +160,7 @@ void SystemInformation::lsbReleaseFinished()
     }
 
     setBugzillaPlatform(platform);
+    m_complete = true;
 }
 
 //this function maps the distribution information to an "Hardware Platform"    .
@@ -122,91 +168,45 @@ void SystemInformation::lsbReleaseFinished()
 //side, they need to be updated here as well                                   .
 QString SystemInformation::guessBugzillaPlatform(const QString& distroInfo) const
 {
-    if ( distroInfo.contains(QStringLiteral("suse"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("openSUSE RPMs"));
-    } else if ( distroInfo.contains(QStringLiteral("mint"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("Mint (Ubuntu Based)"));
-    } else if ( distroInfo.contains(QStringLiteral("lmde"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("Mint (Debian Based)"));
-    } else if ( distroInfo.contains(QStringLiteral("ubuntu"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("Ubuntu Packages"));
-    } else if ( distroInfo.contains(QStringLiteral("fedora"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("Fedora RPMs"));
-    } else if ( distroInfo.contains(QStringLiteral("redhat"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("RedHat RPMs"));
-    } else if ( distroInfo.contains(QStringLiteral("gentoo"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("Gentoo Packages"));
-    } else if ( distroInfo.contains(QStringLiteral("mandriva"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("Mandriva RPMs"));
-    } else if ( distroInfo.contains(QStringLiteral("mageia"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("Mageia RPMs"));
-    } else if ( distroInfo.contains(QStringLiteral("slack"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("Slackware Packages"));
-    } else if ( distroInfo.contains(QStringLiteral("pclinuxos"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("PCLinuxOS"));
-    } else if ( distroInfo.contains(QStringLiteral("pardus"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("Pardus Packages"));
-    } else if ( distroInfo.contains(QStringLiteral("freebsd"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("FreeBSD Ports"));
-    } else if ( distroInfo.contains(QStringLiteral("netbsd"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("NetBSD pkgsrc"));
-    } else if ( distroInfo.contains(QStringLiteral("openbsd"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("OpenBSD Packages"));
-    } else if ( distroInfo.contains(QStringLiteral("solaris"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("Solaris Packages"));
-    } else if ( distroInfo.contains(QStringLiteral("chakra"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("Chakra"));
-    } else if ( distroInfo.contains(QStringLiteral("ms windows"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("MS Windows"));
-    } else if ( distroInfo.contains(QStringLiteral("arch"),Qt::CaseInsensitive) ) {
-        return (QLatin1String("Archlinux Packages"));
-    } else if ( distroInfo.contains(QStringLiteral("debian"),Qt::CaseInsensitive) ) {
-        if ( distroInfo.contains(QStringLiteral("unstable"),Qt::CaseInsensitive) ) {
-            return (QLatin1String("Debian unstable"));
-        } else if ( distroInfo.contains(QStringLiteral("testing"),Qt::CaseInsensitive) ) {
-            return (QLatin1String("Debian testing"));
-        } else {
-            return (QLatin1String("Debian stable"));
+    static QHash<QString, QString> platforms {
+        { QStringLiteral("suse"), QStringLiteral("openSUSE RPMs") },
+        { QStringLiteral("mint"), QStringLiteral("Mint (Ubuntu Based)") },
+        { QStringLiteral("lmde"), QStringLiteral("Mint (Debian Based)") },
+        { QStringLiteral("ubuntu"), QStringLiteral("Ubuntu Packages") },
+        { QStringLiteral("fedora"), QStringLiteral("Fedora RPMs") },
+        { QStringLiteral("redhat"), QStringLiteral("RedHat RPMs") },
+        { QStringLiteral("gentoo"), QStringLiteral("Gentoo Packages") },
+        { QStringLiteral("mandriva"), QStringLiteral("Mandriva RPMs") },
+        { QStringLiteral("mageia"), QStringLiteral("Mageia RPMs") },
+        { QStringLiteral("slack"), QStringLiteral("Slackware Packages") },
+        { QStringLiteral("pclinuxos"), QStringLiteral("PCLinuxOS") },
+        { QStringLiteral("pardus"), QStringLiteral("Pardus Packages") },
+        { QStringLiteral("freebsd"), QStringLiteral("FreeBSD Ports") },
+        { QStringLiteral("netbsd"), QStringLiteral("NetBSD pkgsrc") },
+        { QStringLiteral("openbsd"), QStringLiteral("OpenBSD Packages") },
+        { QStringLiteral("solaris"), QStringLiteral("Solaris Packages") },
+        { QStringLiteral("chakra"), QStringLiteral("Chakra") },
+        { QStringLiteral("ms windows"), QStringLiteral("MS Windows") },
+        { QStringLiteral("arch"), QStringLiteral("Archlinux Packages") }
+    };
+    for (auto it = platforms.constBegin(); it != platforms.constEnd(); ++it) {
+        if (distroInfo.contains(it.key(), Qt::CaseInsensitive)) {
+            return it.value();
         }
-    } else {
-        return PLATFORM_UNSPECIFIED;
     }
-}
 
-//this function maps the operating system to an OS value that is accepted by bugs.kde.org.
-//if the values change on the server side, they need to be updated here as well.
-QString SystemInformation::fetchOSBasicInformation() const
-{
-    //krazy:excludeall=cpp
-    //Get the base OS string (bugzillaOS)
-#if defined(Q_OS_LINUX)
-    return QLatin1String("Linux");
-#elif defined(Q_OS_FREEBSD)
-    return QLatin1String("FreeBSD");
-#elif defined(Q_OS_NETBSD)
-    return QLatin1String("NetBSD");
-#elif defined(Q_OS_OPENBSD)
-    return QLatin1String("OpenBSD");
-#elif defined(Q_OS_AIX)
-    return QLatin1String("AIX");
-#elif defined(Q_OS_HPUX)
-    return QLatin1String("HP-UX");
-#elif defined(Q_OS_IRIX)
-    return QLatin1String("IRIX");
-#elif defined(Q_OS_OSF)
-    return QLatin1String("Tru64");
-#elif defined(Q_OS_SOLARIS)
-    return QLatin1String("Solaris");
-#elif defined(Q_OS_CYGWIN)
-    return QLatin1String("Cygwin");
-#elif defined(Q_OS_DARWIN)
-    return QLatin1String("OS X");
-#elif defined(Q_OS_WIN32)
-    return QLatin1String("MS Windows");
-#else
-    return OS_UNSPECIFIED;
-#endif
+    // Debian has multiple platforms.
+    if (distroInfo.contains(QLatin1String("debian"), Qt::CaseInsensitive)) {
+        if (distroInfo.contains(QLatin1String("unstable"), Qt::CaseInsensitive)) {
+            return QStringLiteral("Debian unstable");
+        } else if (distroInfo.contains(QLatin1String("testing"), Qt::CaseInsensitive)) {
+            return QStringLiteral("Debian testing");
+        } else {
+            return QStringLiteral("Debian stable");
+        }
+    }
 
+    return PLATFORM_UNSPECIFIED;
 }
 
 QString SystemInformation::fetchOSDetailInformation() const
@@ -216,7 +216,13 @@ QString SystemInformation::fetchOSDetailInformation() const
 
 #if HAVE_UNAME
     struct utsname buf;
-    if (uname(&buf) == -1) {
+
+    auto unameFunc = &uname;
+    if (m_infoConfig.unameFunc) {
+        unameFunc = (int (*)(utsname*))m_infoConfig.unameFunc;
+    }
+
+    if ((*unameFunc)(&buf) == -1) {
         qCDebug(DRKONQI_LOG) << "call to uname failed" << errno;
     } else {
         operatingSystem = QString::fromLocal8Bit(buf.sysname) + QLatin1Char(' ')
@@ -228,33 +234,10 @@ QString SystemInformation::fetchOSDetailInformation() const
     return operatingSystem;
 }
 
-QString SystemInformation::fetchOSReleaseInformation() const
+QString SystemInformation::fetchOSReleaseInformation()
 {
-    QFile data(QStringLiteral("/etc/os-release"));
-    if (!data.open(QIODevice::ReadOnly | QIODevice::Text)) {
-        return QString();
-    }
-
-    QMap<QString,QString> distroInfos;
-
-    QTextStream in(&data);
-    while (!in.atEnd()) {
-        const QString line = in.readLine();
-
-        // its format is one simple NAME=VALUE per line
-        // don't use QString.split() here since its value might contain '=''
-        const int index = line.indexOf(QLatin1Char('='));
-        if ( index != -1 ) {
-            const QString key = line.left(index);
-            const QString value = line.mid(index+1);
-            distroInfos.insert(key, value);
-        }
-    }
-
-    // the PRETTY_NAME entry should be the most appropriate one,
-    // but I could be wrong.
-    const QString prettyName = distroInfos.value(QStringLiteral("PRETTY_NAME"), QStringLiteral("Linux"));
-    return prettyName;
+    KOSRelease os(m_infoConfig.osReleasePath);
+    return m_distributionPrettyName = os.prettyName();
 }
 
 QString SystemInformation::operatingSystem() const
@@ -277,9 +260,9 @@ void SystemInformation::setBugzillaPlatform(const QString & platform)
     m_bugzillaPlatform = platform;
 }
 
-QString SystemInformation::lsbRelease() const
+QString SystemInformation::distributionPrettyName() const
 {
-    return m_lsbRelease;
+    return m_distributionPrettyName;
 }
 
 bool SystemInformation::compiledSources() const
@@ -301,3 +284,8 @@ QString SystemInformation::frameworksVersion() const
 {
     return KCoreAddons::versionString();
 }
+
+bool SystemInformation::complete() const
+{
+    return m_complete;
+}
